@@ -1,6 +1,9 @@
+import platform
 import websocket
 import json
 import time
+import re
+import importlib
 from threading import Thread
 from .utils.events import Events
 from .utils.http import *
@@ -10,13 +13,15 @@ from .objects.guild import Guild
 from .objects.user import User
 from .objects.activity import Activity
 from .objects.presence import Presence
+from .command.command import Command
 
 class Fastcord:
 
     api = "https://discordapp.com/api"
 
-    def __init__(self, token, verbose=False):
+    def __init__(self, token, prefix=None, verbose=False):
         self.token = token
+        self.prefix = prefix
         self.verbose = verbose
         self.ws = websocket.WebSocketApp("wss://gateway.discord.gg/?v=6&encoding=json",
             on_message=lambda ws, msg: self.on_message(ws, msg), on_close=lambda ws: self.on_close(ws))
@@ -28,6 +33,7 @@ class Fastcord:
         self.ready = False
         self.events = Events(verbose)
         self.on_event = self.events.on_event()
+        self.commands = []
     
     def run(self):
         self.ws.run_forever()
@@ -78,6 +84,12 @@ class Fastcord:
     
     def get_channel(self, channel_id):
         return Channel(self, get(f"{self.api}/channels/{channel_id}", { "Authorization": "Bot " + self.token }))
+
+    def load_command(self, command):
+        self.commands.append(command)
+    
+    def invalid_usage(self, msg, cmd, args):
+        msg.channel.send(f"Usage: {cmd} {' '.join(args)}")
     
     def on_message(self, ws, msg):
         if self.verbose:
@@ -104,7 +116,58 @@ class Fastcord:
                 self.events.call("ready")
             
             if msg["t"] == "MESSAGE_CREATE":
-                self.events.call("message", Message(self, msg["d"]))
+                msg = Message(self, msg["d"])
+
+                if not self.prefix:
+                    s = re.split("\s+", msg.content)
+
+                    if s[0] == f"<@!{self.id}>":
+                        for command in self.commands:
+                            if command.name == s[1]:
+                                if len(command.args) != 0:
+                                    args = [t.strip('"') for t in re.findall(r'[^\s"]+|"[^"]*"', ' '.join(s[2:]))]
+
+                                    if len(args) < len(command.args):
+                                        self.invalid_usage(msg, s[1], command.args)
+                                        break
+
+                                    _args = {}
+
+                                    for i in range(len(command.args)):
+                                        _args[command.args[i]] = args[i]
+
+                                    command.invoke(self, msg, _args)
+                                    break;
+
+                                command.invoke(self, msg, None)
+                                break
+                else:
+                    s = re.split("\s+", msg.content)
+
+                    if msg.content.startswith(self.prefix):
+                        cmd = s[0].replace(self.prefix, "", 1)
+
+                        for command in self.commands:
+                            if command.name == cmd:
+                                if len(command.args) != 0:
+                                    args = [t.strip('"') for t in re.findall(r'[^\s"]+|"[^"]*"', ' '.join(s[1:]))]
+
+                                    if len(args) < len(command.args):
+                                        self.invalid_usage(msg, cmd, command.args)
+                                        break
+
+                                    _args = {}
+
+                                    for i in range(len(command.args)):
+                                        _args[command.args[i]] = args[i]
+
+                                    command.invoke(self, msg, _args)
+                                    break;
+                                
+                                command.invoke(self, msg, None)
+                                break
+
+                self.events.call("message", msg)
 
         if msg["op"] == 9: # opcode 9 invalid session
             time.sleep(5)
@@ -116,7 +179,7 @@ class Fastcord:
                 "d": {
                     "token": self.token,
                     "properties": {
-                        "$os": "linux",
+                        "$os": platform.system(),
                         "$browser": "fastcord",
                         "$device": "fastcord"
                     }
@@ -134,7 +197,7 @@ class Fastcord:
                 "d": {
                     "token": self.token,
                     "properties": {
-                        "$os": "linux",
+                        "$os": platform.system(),
                         "$browser": "fastcord",
                         "$device": "fastcord"
                     }
